<?php
class S2Base_CommandLauncherFactory {
    public static function create($classFiles) {
        $launcher = new S2Base_CommandLauncher();
        foreach ($classFiles as $classFile) {
            $fileInfo = pathinfo($classFile);
            if (strtolower($fileInfo['extension']) == 'php' and 
                preg_match("/^(\w+)/",$fileInfo['basename'],$matches)) {
                $cmdClassName = $matches[1];
                if (!class_exists($cmdClassName,false)) {
                    require_once($classFile);
                }
                if (!class_exists($cmdClassName,false)) {
                    continue;
                }
                $cmdObj = new $cmdClassName();
                if ($cmdObj instanceof S2Base_GenerateCommand) {
                    $launcher->addCommand($cmdObj);
                }
            }
        }
        return $launcher;
    }
}

class S2Base_CommandLauncher {
    private $commands = array();
    public function addCommand(S2Base_GenerateCommand $command){
        $this->commands[$command->getName()] = $command;
    }
    public function main(){
        $cmds = array_keys($this->commands);
        sort($cmds);
        while(true){
            $cmd = S2Base_StdinManager::getValueFromArray($cmds,"Command list");
            if($cmd == S2Base_StdinManager::EXIT_LABEL){
                break;
            }else{
                $this->commands[$cmd]->execute();
            }
        }
    }
}

class S2Base_CommandUtil {
    public static function createDirectory($path){
        if(!file_exists($path)){
            if(!mkdir($path)){
               throw new Exception("Cannot make dir [ $path ]");
            }
            return true;
        }else{
            return false;
        }
    }
    public static function readFile($src){
        if(!is_readable($src)){
            throw new Exception("Cannot read file [ $src ]");
        }
        return file_get_contents($src);
    }
    public static function writeFile($filename,$content){
        if (file_exists($filename)) {
            throw new S2Base_FileExistsException("Already exists. [ $filename ]");
        }
        if(!file_put_contents($filename,$content,LOCK_EX)){
            throw new Exception("Cannot write to file [ $filename ]");
        }
    }    
    public static function getModuleName(){
        $modules = self::getAllModules();
        if(count($modules) == 0){
            throw new Exception("Module not found at all.");
        }
        return S2Base_StdinManager::getValueFromArray($modules,"Module list");
    }
    public static function getAllModules(){
        $entries = scandir(S2BASE_PHP5_APP_DIR . "modules");
        $modules = array();
        foreach($entries as $entry) {
            $path = S2BASE_PHP5_MODULES_DIR . $entry;
            if(!preg_match("/^\./",$entry) and is_dir($path)){
                array_push($modules,$entry);
            }
        }
        return $modules;
    }        
    public static function validate($name,$msg){
        if(!preg_match("/^\w+$/",$name)){
           throw new Exception($msg);
        }   
    }
}

interface S2Base_GenerateCommand {
    public function getName();
    public function execute();
}

class S2Base_StdinManager {
    const EXIT_LABEL = "(exit)";
    public static function getValueFromArray($cmds, $title){
        $cmds = array_merge(array(self::EXIT_LABEL),$cmds);
        print "\n[ $title ]\n";
        $number = null;
        while(true){
            foreach($cmds as $key=>$module){
                print "$key : $module\n";
            }
            print "choice ? : ";
            if(fscanf(STDIN, "%d\n", $number) and
                array_key_exists($number,$cmds)){
                break;
            }
        }
        return $cmds[$number];
    }
    public static function getValue($msg){
        print "\n$msg";
        $val = trim(fgets(STDIN));
        return $val;
    }
    public static function isYes($msg){
        $ret = false;
        while(true){
            print "\n$msg (y/n) : ";
            $val = trim(fgets(STDIN));
            if(strcasecmp($val,'y') == 0){
                $ret = true;
                break;
            }else if (strcasecmp($val,'n') == 0){
                $ret = false;
                break;
            }
        }
        return $ret;
    }
}

class S2Base_FileExistsException extends Exception {
    public function __construct($msg){
        parent::__construct($msg);
    }   
}

?>
